<?php

namespace Drupal\Tests\access_by_taxonomy\Kernel;

use Drupal\user\Entity\Role;

/**
 * Class InstallTest tests that installing the module works fine.
 *
 * @package Drupal\Tests\access_by_taxonomy\Kernel
 */
class InstallTest extends AccessByTaxonomyKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'taxonomy',
    'node',
    'user',
    'text',
    'field',
    'system',
    'access_by_taxonomy',
    'language',
    'content_translation',
  ];

  /**
   * Test that upon install, roles are populated.
   */
  public function testRolesPopulated() {
    $roles = \Drupal::service('entity_type.manager')
      ->getStorage('user_role')
      ->loadMultiple();
    foreach ($roles as $role) {
      // When running tests for some reason anonymous doesn't have an ID.
      if ($role->id() === NULL) {
        \Drupal::service('access_by_taxonomy.role_grants')->insertRoleGrant('anonymous');
      }
    }
    $results = \Drupal::service('access_by_taxonomy.role_grants')->getRoleGrant();
    $this->assertCount(count($roles), $results);
  }

  /**
   * Tests that fields are added to existing taxonomies.
   */
  public function testFieldsArePresent() {
    $fields = $this->container->get('entity_field.manager')->getFieldDefinitions('taxonomy_term', 'test');
    $this->assertTrue(!empty($fields['field_allowed_users']));
    $this->assertTrue(!empty($fields['field_allowed_roles']));

    $fields = $this->container->get('entity_field.manager')->getFieldDefinitions('taxonomy_term', 'test2');
    $this->assertTrue(!empty($fields['field_allowed_users']));
    $this->assertTrue(!empty($fields['field_allowed_roles']));
  }

  /**
   * Adds and removes a role, tests grants are updated accordingly.
   */
  public function testRoleAddAndRemove() {
    $current_gids = \Drupal::service('access_by_taxonomy.role_grants')->getRoleGrant();
    $new_role = $this->drupalCreateRole([], 'new_role');
    $has_been_added = \Drupal::service('access_by_taxonomy.role_grants')->getRoleGrant();
    $this->assertTrue(count($current_gids) + 1 === count($has_been_added));
    $role = Role::load($new_role);
    $role->delete();
    $has_been_removed = \Drupal::service('access_by_taxonomy.role_grants')->getRoleGrant();
    $this->assertTrue(count($current_gids) === count($has_been_removed));
  }

}
